# The spelling of the command used in LUKIS stream

#The followings are just like atoms
# pdf generic commands
LineTo = "LineTo"
MoveTo = "MoveTo"
Stroke = "Stroke"
Close = "Close"
Close_Stroke = "Close_Stroke"
Fill = "Fill"
Fill_Stroke = "Fill_Stroke"
Close_Fill_Stroke = "Close_Fill_Stroke"
Scale = "Scale"
Translate = "Translate"
CurveTo = "CurveTo"
RotateCoordinate = "RotateCoordinate"
SetLineWidth = "SetLineWidth"
SetDash = "SetDash"
SetLineJoin = "SetLineJoin"
SetLineColor = "SetLineColor"
SetFillColor = "SetFillColor"
SetFont = "SetFont"
Text= "Text"
Picture = "Picture"
